'use strict';

const _ = require('lodash');
const path = require('path');
const {createLogger, format, transports, addColors} = require('winston');
const DailyRotateFile = require('winston-daily-rotate-file');
const {processConfig, rcConfig, baseConfig} = require('./config');
const {pretty, colorize} = require('./format');

/**
 * create logger
 * @param {string | {}} loggerOptions
 */
module.exports = (loggerOptions = {}) => {
    // config
    if (typeof loggerOptions === 'string')
        loggerOptions = {name: loggerOptions};
    const options = _.defaults(
        loggerOptions,
        processConfig,
        rcConfig,
        baseConfig,
    );

    const logger = createLogger({
        level: options.baseLevel.toLowerCase(),
        levels: options.loggerLeverls.levels,
        format: format.combine(
            format.splat(),
            format.timestamp(),
            pretty(options),
        ),
    });
    addColors(options.loggerLeverls.colors);

    if (options.console) logger.add(createConsoleLog(options));
    if (options.rotate) logger.add(createRotateLog(options));
    return logger;
};

// Helpers

function createConsoleLog(options) {
    return new transports.Console({
        level: options.level.toLowerCase(),
        format: colorize(options),
    });
}

function createRotateLog(options) {
    const file = path.join(
        options.dirname,
        options.filename || options.toFilename(options.name),
    );
    return new DailyRotateFile({
        level: options.dailyLevel.toLowerCase(),
        dirname: path.dirname(file),
        filename: path.basename(file),
        maxSize: options.maxSize,
        maxFiles: options.maxFiles,
    });
}
